#include "view/hover_window.h"
#include "view/source_wgt.h"
#include "$(appName).h"

static struct {
	Evas_Object *label;
	Evas_Object *bg;
} s_view_data = {
	.label = NULL,
	.bg = NULL
};

static Evas_Object *
__create_label(Evas_Object *parent);

static Eina_Bool
__text_paste_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev);

Evas_Object *
_create_hover_window(Evas_Object *parent, int width, int height, const char *theme_extension)
{
	/* label: */
	s_view_data.label = __create_label(parent);

	if (!s_view_data.label) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Hover window: label acquiring error");
		return NULL;
	}

	/* set wrapping to word: */
	elm_label_line_wrap_set(s_view_data.label, ELM_WRAP_WORD);
	elm_label_ellipsis_set(s_view_data.label, EINA_TRUE);

	evas_object_move(s_view_data.label, 0, 0);
	evas_object_resize(s_view_data.label, width, height);

	/* add some background: */
	s_view_data.bg = elm_bg_add(s_view_data.label);

	if (!s_view_data.bg) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Hover window: background acquiring error");
		return NULL;
	}

	elm_bg_color_set(s_view_data.bg, 0, 150, 255);
	evas_object_resize(s_view_data.bg, width, height);

	/* This is how we use 'cnp_custom 'theme extension from edje file: */
	elm_theme_extension_add(NULL, theme_extension);
	elm_object_style_set(s_view_data.label, "cnp_custom");

	evas_object_layer_set(s_view_data.label, EVAS_LAYER_MAX);

	return s_view_data.label;
}

void
_show_hover_window(void)
{
	evas_object_show(s_view_data.bg);
	evas_object_show(s_view_data.label);
}

void
_hide_hover_window(void)
{
	evas_object_hide(s_view_data.bg);
	evas_object_hide(s_view_data.label);
}

void
_set_hover_window_coordinates(int x, int y)
{
	Evas_Coord w = 0, h = 0;
	evas_object_geometry_get(s_view_data.label, NULL, NULL, &w, &h);

	evas_object_move(s_view_data.bg, (Evas_Coord)(x - ((int)w)/2), (Evas_Coord)(y - ((int)h)/2));
	evas_object_move(s_view_data.label, (Evas_Coord)(x - ((int)w)/2), (Evas_Coord)(y - ((int)h)/2));
}

void
_paste_hover_window_text(const char *text_to_paste)
{
	/* We could just use elm_object_text_set, but for example purpose let's use cnp mechanism: */

	/* use copy mechanism, fill copy buffer associated with the label: */
	elm_cnp_selection_set(s_view_data.label, ELM_SEL_TYPE_PRIMARY, ELM_SEL_FORMAT_TEXT, text_to_paste, strlen(text_to_paste));

	/* get the text from the clipboard and since */
	/*  we use 'Label' widget type, we have to use custom drop callback: _text_paste_cb */
	elm_cnp_selection_get(s_view_data.label, ELM_SEL_TYPE_PRIMARY, ELM_SEL_FORMAT_TEXT, __text_paste_cb, NULL);
}

const char *
get_hover_window_text(void)
{
	return elm_object_text_get(s_view_data.label);
}

static Evas_Object *
__create_label(Evas_Object *parent)
{
	return _source_wgt_create(parent);
}

static Eina_Bool
__text_paste_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev)
{
	/* obj - our hovering label */
	/* ev - information about selection */

	if (!ev || ev->format != ELM_SEL_FORMAT_TEXT)
		return EINA_FALSE;

	dlog_print(DLOG_INFO, LOG_TAG, "Copying selection to hover window: %s", ev->data);

	elm_object_text_set(obj, (const char *)ev->data);

	return EINA_TRUE;
}
